(******************************************************************************)
(*   __               ___     CAVE: Concurrent Algorithm VErifier             *)
(*  /     /\  \    / |        Copyright (c) Viktor Vafeiadis                  *)
(* |     /--\  \  /  |---                                                     *)
(*  \__ /    \  \/   |___     See LICENSE.txt for license.                    *)
(*                                                                            *)
(******************************************************************************)
(** Translate parse tree into the intermediate abstract syntax tree *)
open Misc
open Parsetree
open Exp
open Assertions
open Commands

exception Undef_non_pure_expression 
exception Conversion_error of string * Location.t

type fun_info_ht = (string,fun_info) Hashtbl.t

(** Convert a [p_expression] to a [exp]. 
    Throws [Undef_non_pure_expression] if the expression contains indirections.
*)
val convert_exp : p_expression -> exp

(** [convert_exp_se fun_ht pe stmts] returns the read side-effects required for
   the indirections in [pe] and a {!exp} representing [pe] *)
val convert_exp_se : 
  fun_info_ht ->
  component list -> p_expression -> can_stmt list -> can_stmt list * exp

(** [convert_exp_list_se fun_ht pel stmts] returns the read side-effects
   required for the indirections in [pel] and a {!exp list} representing
   [pel] *) 
val convert_exp_list_se :
  fun_info_ht ->
  component list -> p_expression list -> can_stmt list -> can_stmt list * exp list

val convert_read_memory_order :
  p_read_memory_order -> read_memory_order

val convert_cas_memory_order_se :
  fun_info_ht -> component list -> p_cas_memory_order -> can_stmt list ->
  can_stmt list * cas_memory_order

val convert_write_memory_order_se :
  fun_info_ht -> component list -> p_write_memory_order -> can_stmt list ->
  can_stmt list * write_memory_order

(** [convert_assume_exp fun_ht e] returns the statements required to evaluate 
   [assume e] and [assume (not e)]. *)
val convert_assume_exp :
  fun_info_ht ->
  component list -> p_expression -> can_stmt list * can_stmt list

val convert_prop : a_proposition -> Location.t -> cprop
val prop_to_ip_body : a_proposition -> Location.t -> (Pure.t * cform) list 

